@@ -16,6 +16,7 @@ from . import config
 from . import mapping as mapp
 from . import util
 #from . import io_tools
+#from .tools import xarray
             
 class Datatable(pd.DataFrame):
     
@@ -37,9 +38,15 @@ class Datatable(pd.DataFrame):
         except:
             self.ID = None
 
-        #compatibililty with v1
-#        if not 'model' in self.meta.keys():
-#            self.meta['model'] = ''
+    
+        if config.AVAILABLE_XARRAY:
+            self.to_xarray = self._to_xarray
+    
+    
+    
+    def _to_xarray(self):
+        
+        return core.xr.DataArray(self.values, coords=[self.index, self.columns], dims=['space','time'], attrs=self.meta)
     
     @property
     def _constructor(self):
@@ -88,10 +95,18 @@ class Datatable(pd.DataFrame):
             data = data.copy(deep=True)
         return Datatable(data).__finalize__(self) 
 
+    def diff(self, periods=1, axis=0):
+        
+        out = super(Datatable, self).diff(periods=periods, axis=axis)
+        out.meta['unit'] = self.meta['unit']
+        
+        return out
     
 #with pd.ExcelWriter('the_file.xlsx', engine='openpyxl', mode='a') as writer: 
 #     data_filtered.to_excel(writer) 
-
+    x = pd.DataFrame()
+    x.diff()
+    
     def to_excel(self, fileName = None, sheetName = "Sheet0", writer = None, append=False):
         if writer is None:
             if append:
@@ -122,7 +137,7 @@ class Datatable(pd.DataFrame):
         else:
             assert fileName[-4:]  == '.csv'
         
-        fid = open(fileName,'w')
+        fid = open(fileName,'w', encoding='utf-8')
         fid.write(config.META_DECLARATION)
         
         for key, value in sorted(self.meta.items()):
@@ -136,10 +151,13 @@ class Datatable(pd.DataFrame):
         
     def convert(self, newUnit, context=None):
         
+        if self.meta['unit'] == newUnit:
+            return self
+        
         dfNew = self.copy()
 #        oldUnit = core.getUnit(self.meta['unit'])
 #        factor = (1* oldUnit).to(newUnit).m
-
+        
         factor = core.conversionFactor(self.meta['unit'], newUnit, context)
         
         dfNew.loc[:] =self.values * factor
@@ -147,39 +165,25 @@ class Datatable(pd.DataFrame):
         dfNew.meta['modified'] = core.getTimeString()
         return dfNew
         
-    
-    
-    def interpolate(self, method, newSource):
+    def aggregate_region(self, mapping):
+        """ 
+        This functions added the aggregates to the table according to the provided
+        mapping.( See datatools.mapp.regions)
         
-        if method == 'linear':
-            from scipy import interpolate
-            import numpy as np
-            xData = self.columns.values.astype(float)
-            yData = self.values
-            
-            for row in yData:
-                idxNan = np.isnan(row)
-                interpolator = interpolate.interp1d(xData[~idxNan], row[~idxNan], kind='linear')
-                row[idxNan] = interpolator(xData[idxNan])
-        else:
-            raise(NotImplementedError())
-            
-        dfNew = self.copy()
-        dfNew.loc[:] = yData
+        Returns the result, but does not inplace add it.
+        """
+        from datatoolbox.tools.for_datatables import aggregate_region
+        return aggregate_region(self, mapping)
+    
+    def interpolate(self, method="linear", add_missing_years=False):
+        from datatoolbox.tools.for_datatables import interpolate
         
-        dfNew.meta['modified'] = core.getTimeString()
-        dfNew.meta['source'] = newSource
-        return dfNew
-
-#    def filter(self, context, regionID):
-#        """ 
-#        valid filter aguments are:
-#            "region"
-#        """
-#        members = mapp.getMembersOfRegion(context, regionID)
-#        mask = self.index.isin(members)
-#
-#        return self.iloc[mask,:]
+        if add_missing_years:
+            for col in list(range(self.columns.min(),self.columns.max()+1)):
+                if col not in self.columns:
+                    self.loc[:,col] = np.nan
+            self = self.loc[:,list(range(self.columns.min(),self.columns.max()+1))]
+        return interpolate(self, method)
     
     def clean(self):
 
@@ -267,18 +271,21 @@ class Datatable(pd.DataFrame):
     
         
     def __add__(self, other):
-        #print('other in' + str(other))
         if isinstance(other,Datatable):
             
-            factor = core.getUnit(other.meta['unit']).to(self.meta['unit']).m
-            #print('factor: ' + str(factor))
-            #print('other' + str(other))
-            #print(other * factor)
-            out = Datatable(super(Datatable, self).__add__(other * factor))
+            if self.meta['unit'] == other.meta['unit']:
+                factor = 1
+            else:
+                factor = core.getUnit(other.meta['unit']).to(self.meta['unit']).m
+            
+            rhs = pd.DataFrame(other * factor)
+            out = Datatable(super(Datatable, self.copy()).__add__(rhs))
+
             out.meta['unit'] = self.meta['unit']
             out.meta['source'] = 'calculation'
         else:
-            
+#            import pdb
+#            pdb.set_trace()
             out = Datatable(super(Datatable, self).__add__(other))
             out.meta['unit'] = self.meta['unit']
             out.meta['source'] = 'calculation'
@@ -288,8 +295,12 @@ class Datatable(pd.DataFrame):
     
     def __sub__(self, other):
         if isinstance(other,Datatable):
-            factor = core.getUnit(other.meta['unit']).to(self.meta['unit']).m
-            out = Datatable(super(Datatable, self).__sub__(other * factor))
+            if self.meta['unit'] == other.meta['unit']:
+                factor = 1
+            else:
+                factor = core.getUnit(other.meta['unit']).to(self.meta['unit']).m
+            rhs = pd.DataFrame(other * factor)
+            out = Datatable(super(Datatable, self).__sub__(rhs))
             out.meta['unit'] = self.meta['unit']
             out.meta['source'] = 'calculation'
         else:
@@ -297,10 +308,13 @@ class Datatable(pd.DataFrame):
             out.meta['unit'] = self.meta['unit']
             out.meta['source'] = 'calculation'
         return out
-
+    
     def __rsub__(self, other):
         if isinstance(other,Datatable):
-            factor = core.getUnit(other.meta['unit']).to(self.meta['unit']).m
+            if self.meta['unit'] == other.meta['unit']:
+                factor = 1
+            else:
+                factor = core.getUnit(other.meta['unit']).to(self.meta['unit']).m
             out = Datatable(super(Datatable, self).__rsub__(other * factor))
             out.meta['unit'] = self.meta['unit']
             out.meta['source'] = 'calculation'
@@ -379,24 +393,33 @@ class Datatable(pd.DataFrame):
     def _repr_html_(self):
         outStr = """"""
         if 'ID' in self.meta.keys():
-            outStr += '=== Datatable - ' + self.meta['ID'] + ' ===\n'
+            outStr += '=== Datatable - ' + self.meta['ID'] + ' ===<br/>\n'
         else:
-            outStr += '=== Datatable ===\n'
+            outStr += '=== Datatable ===<br/>\n'
         for key in self.meta.keys():
             if self.meta[key] is not None:
-                outStr += key + ': ' + str(self.meta[key]) + ' \n'
+                outStr += key + ': ' + str(self.meta[key]) + ' <br/>\n'
         outStr += super(Datatable, self)._repr_html_()
         return outStr
-
+#%%
 class TableSet(dict):
     def __init__(self, IDList=None):
         super(dict, self).__init__()
-        self.inventory = pd.DataFrame(columns = config.INVENTORY_FIELDS)
+        self.inventory = pd.DataFrame(columns = ['key']+ config.INVENTORY_FIELDS)
         
         if IDList is not None:
             for tableID in IDList:
                 self.add(core.DB.getTable(tableID))
-    
+        
+        
+#        if config.AVAILABLE_XARRAY:
+#           self.to_Xarray = self._to_Xarray         
+            
+    def to_xarray(self, dimensions):
+        if not config.AVAILABLE_XARRAY:
+            raise(BaseException('xarray not available'))
+        return xarray.to_XDataArray(self, dimensions)
+        
     def __iter__(self):
         return iter(self.values())
     
@@ -418,16 +441,51 @@ class TableSet(dict):
             if datatable.ID is None:
                 datatable.generateTableID()
             self[datatable.ID] = datatable
-            self.inventory.loc[datatable.ID] = [datatable.meta.get(x,None) for x in config.INVENTORY_FIELDS]
+            self.inventory.loc[datatable.ID, "key"] = datatable.ID
+            self.inventory.loc[datatable.ID, config.INVENTORY_FIELDS] = [datatable.meta.get(x,None) for x in config.INVENTORY_FIELDS]
     
     def remove(self, tableID):
         del self[tableID]
         self.inventory.drop(tableID, inplace=True)
         
     
-    def filter(self, ):
-        pass
-    
+    def filter(self,**kwargs):
+        
+        inv = self.inventory.copy()
+        for key in kwargs.keys():
+            #table = table.loc[self.inventory[key] == kwargs[key]]
+            mask = self.inventory[key].str.contains(kwargs[key], regex=False)
+            mask[pd.isna(mask)] = False
+            mask = mask.astype(bool)
+            inv = inv.loc[mask].copy()
+            
+        newTableSet = TableSet()
+        for key in inv.index:
+            newTableSet[key] = self[key]
+            
+        return newTableSet
+#        def getInventory(self, **kwargs):
+#        
+#        table = self.inventory.copy()
+#        for key in kwargs.keys():
+#            #table = table.loc[self.inventory[key] == kwargs[key]]
+#            mask = self.inventory[key].str.contains(kwargs[key], regex=False)
+#            mask[pd.isna(mask)] = False
+#            mask = mask.astype(bool)
+#            table = table.loc[mask].copy()
+#            
+#        
+#        return table
+    def aggregate_to_region(self, mapping):
+        """ 
+        This functions added the aggregates to the output according to the provided
+        mapping.( See datatools.mapp.regions)
+        
+        Returns the result, but does not inplace add it.
+        """
+        return util.aggregate_tableset_to_region(self, mapping)
+        
+        
     def __getitem__(self, key):
         item = super(TableSet, self).__getitem__(key)
         
@@ -437,13 +495,30 @@ class TableSet(dict):
             self[key] = item
         
         return item
+
+    def __setitem__(self, key, datatable):
+        super(TableSet, self).__setitem__(key, datatable)
+        
+        if datatable.ID is None:
+            datatable.generateTableID()
+        self.inventory.loc[datatable.ID, "key"] = key
+        self.inventory.loc[datatable.ID, config.INVENTORY_FIELDS] = [datatable.meta.get(x,None) for x in config.INVENTORY_FIELDS]
     
-    def to_excel(self, fileName):
-        writer = pd.ExcelWriter(fileName, 
-            engine='openpyxl', 
-            mode='a',
-            datetime_format='mmm d yyyy hh:mm:ss',
-            date_format='mmmm dd yyyy') 
+
+    
+    def to_excel(self, fileName, append=False):
+       
+
+        if append:
+            writer = pd.ExcelWriter(fileName, 
+                                    engine='openpyxl', mode='a',
+                                    datetime_format='mmm d yyyy hh:mm:ss',
+                                    date_format='mmmm dd yyyy')  
+        else:
+            writer = pd.ExcelWriter(fileName,
+                                    engine='xlsxwriter',
+                                    datetime_format='mmm d yyyy hh:mm:ss',
+                                    date_format='mmmm dd yyyy')  
         
         for i,eKey in enumerate(self.keys()):
             table = self[eKey].dropna(how='all', axis=1).dropna(how='all', axis=0)
@@ -483,6 +558,12 @@ class TableSet(dict):
             
         return coTables
 
+    def variables(self):
+        return list(self.inventory.variable.unique())
+
+    def pathways(self):
+        return list(self.inventory.pathway.unique())
+
     def entities(self):
         return list(self.inventory.entity.unique())
 
@@ -500,6 +581,8 @@ class TableSet(dict):
         for key in self.keys():
             table= copy.copy(self[key])
             #print(table.columns)
+            if (len(table.index)== 0) or (len(table.columns)== 0):
+                continue
             
             oldColumns = list(table.columns)
             minMax = (max(minMax[0], min(oldColumns)), min(minMax[1],max(oldColumns)))
@@ -508,15 +591,17 @@ class TableSet(dict):
             
             table.loc[:,'region'] = table.index
             table.loc[:,'unit']   = table.meta['unit']
-            table.loc[:,'variable']   = table.meta['entity']
+            table.loc[:,'variable']   = table.meta['variable']
             
-            scenModel = table.meta['scenario'].split('|')
-            if len(scenModel) == 2:
-                table.loc[:,'model'] = scenModel[1]
-                table.loc[:,'scenario']   =scenModel[0]
+            try:
+                table.loc[:,'scenario'] = table.meta['scenario']
+            except:
+                print(table.meta)
+                raise(BaseException('meta key "scenario" not found'))
+            if 'model' in table.meta.keys():
+                table.loc[:,'model'] = table.meta['model']
             else:
-                table.loc[:,'scenario']   = table.meta['scenario']
-                table.loc[:,'model']      = ''
+                table.loc[:,'model']   = ''
                 
             tableNew = table.loc[:, ['variable', 'region', 'scenario',  'model', 'unit'] +  oldColumns]
             tableNew.index= range(len(tableNew.index))
@@ -539,7 +624,7 @@ class TableSet(dict):
         return fullDf.loc[:, columns]
     
     def to_IamDataFrame(self):
-        #%%
+        
         import pyam
         import copy
         #self = dt.getTables(res.index)
@@ -548,6 +633,8 @@ class TableSet(dict):
         for key in self.keys():
             table= copy.copy(self[key])
             #print(table.columns)
+            if (len(table.index)== 0) or (len(table.columns)== 0):
+                continue
             
             oldColumns = list(table.columns)
             minMax = (max(minMax[0], min(oldColumns)), min(minMax[1],max(oldColumns)))
@@ -561,7 +648,7 @@ class TableSet(dict):
 
             table.loc[:,'scenario'] = table.meta['scenario']
             if 'model' in table.meta.keys():
-                table.loc[:,'model'] = table.meta['scenario']
+                table.loc[:,'model'] = table.meta['model']
             else:
                 table.loc[:,'model']   = ''
             tableNew = table.loc[:, ['variable','region', 'scenario',  'model', 'unit'] +  oldColumns]
@@ -580,7 +667,7 @@ class TableSet(dict):
         return iaDf         
 
     def plotAvailibility(self, regionList= None, years = None):
-        #%%
+        
         avail= 0
         for table in self:
 #            print(table.ID)
@@ -604,7 +691,23 @@ class TableSet(dict):
         plt.colorbar()
         plt.yticks([x +.5 for x in range(len(avail.index))], avail.index)
         plt.xticks([x +.5 for x in range(len(avail.columns))], avail.columns, rotation=45)
-        #%%
+
+
+
+
+#table = dt.getTable(dt.inventory().index[0])
+#table2 = dt.getTable(dt.inventory().index[1])
+#tableset = TableSet()
+#tableset.add(table)
+#tableset['Table1'] = table
+#tableset['Table2'] = table2
+#    
+#print(tableset.pathways())
+#
+#cropsSet = tableset.filter(variable='Crop')
+#%%    
+    
+    
     
 class Visualization():
     """ 
@@ -808,7 +911,7 @@ class Visualization():
                 
 def read_csv(fileName):
     
-    fid = open(fileName,'r')
+    fid = open(fileName,'r', encoding='utf-8')
     
     assert (fid.readline()) == config.META_DECLARATION
     #print(nMetaData)
@@ -821,7 +924,8 @@ def read_csv(fileName):
             break
         dataTuple = line.replace('\n','').split(',')
         meta[dataTuple[0]] = dataTuple[1].strip()
-
+        if "unit" not in meta.keys():
+            meta["unit"] = ""
     df = Datatable(pd.read_csv(fid, index_col=0), meta=meta)
     df.columns = df.columns.map(int)
 
@@ -830,17 +934,39 @@ def read_csv(fileName):
 
 def read_excel(fileName, sheetNames = None):
  
-    out = TableSet()
+    
     if sheetNames is None:
         xlFile = pd.ExcelFile(fileName)
         sheetNames = xlFile.sheet_names
         xlFile.close()
 
+    if len(sheetNames) > 1:
+        out = TableSet()
+        for sheet in sheetNames:
+            fileContent = pd.read_excel(fileName, sheet_name=sheet, header=None)
+            metaDict = dict()
+            try:
+                for idx in fileContent.index:
+                    key, value = fileContent.loc[idx, [0,1]]
+                    if key == '###DATA###':
+                        break
+                    
+                    metaDict[key] = value
+                columnIdx = idx +1
+                dataTable = Datatable(data    = fileContent.loc[columnIdx+1:, 1:].astype(float).values, 
+                                      index   = fileContent.loc[columnIdx+1:, 0], 
+                                      columns = [int(x) for x in fileContent.loc[columnIdx, 1:]], 
+                                      meta    = metaDict)
+                dataTable.generateTableID()
+                out.add(dataTable)
+            except:
+                print('Failed to read the sheet: {}'.format(sheet))
         
-    for sheet in sheetNames:
+    else:
+        sheet = sheetNames[0]
         fileContent = pd.read_excel(fileName, sheet_name=sheet, header=None)
         metaDict = dict()
-        try:
+        if True:
             for idx in fileContent.index:
                 key, value = fileContent.loc[idx, [0,1]]
                 if key == '###DATA###':
@@ -853,9 +979,9 @@ def read_excel(fileName, sheetNames = None):
                                   columns = [int(x) for x in fileContent.loc[columnIdx, 1:]], 
                                   meta    = metaDict)
             dataTable.generateTableID()
-            out.add(dataTable)
-        except:
-            print('Failed to read the sheet: {}'.format(sheet))
+            out = dataTable
+#        except:
+#                print('Failed to read the sheet: {}'.format(sheet))
         
     return out
 #%%
@@ -878,4 +1004,4 @@ if __name__ == '__main__':
     meta['entity'] = 'Emissions|CO2'
     meta['institution'] = 'WDI'
     meta['year']  = '2020'
-    #print(meta)
\ No newline at end of file
+    #print(meta)
