@@ -8,7 +8,7 @@ Created on Fri Mar 22 14:55:32 2019
 
 from . import config
 from .data_structures import Datatable, TableSet, read_csv
-
+from .utilities import plot_query_as_graph
 from . import mapping as mapp
 from . import io_tools as io
 from . import util
@@ -19,7 +19,7 @@ import git
 import tqdm
 import time
 import copy
-
+import types
 
 class Database():
     
@@ -27,8 +27,8 @@ class Database():
         tt = time.time()
         self.path = config.PATH_TO_DATASHELF
         self.gitManager = GitRepository_Manager(config)
-        self.INTVENTORY_PATH = os.path.join(self.path, 'inventory.csv')
-        self.inventory = pd.read_csv(self.INTVENTORY_PATH, index_col=0)
+        self.INVENTORY_PATH = os.path.join(self.path, 'inventory.csv')
+        self.inventory = pd.read_csv(self.INVENTORY_PATH, index_col=0, dtype={'source_year': str})
         self.sources   = self.gitManager.sources
         
         self.gitManager._validateRepository('main')
@@ -71,7 +71,7 @@ class Database():
 
 
     def _reloadInventory(self):
-        self.inventory = pd.read_csv(self.INTVENTORY_PATH, index_col=0)
+        self.inventory = pd.read_csv(self.INVENTORY_PATH, index_col=0, dtype={'source_year': str})
         
     def sourceExists(self, source):
         return source in self.gitManager.sources.index
@@ -99,7 +99,9 @@ class Database():
             mask[pd.isna(mask)] = False
             mask = mask.astype(bool)
             table = table.loc[mask].copy()
-            
+        
+        # test to add function to a instance (does not require class)
+        table.graph = types.MethodType( plot_query_as_graph, table )
         
         return table
 
@@ -112,7 +114,9 @@ class Database():
             mask[pd.isna(mask)] = False
             mask = mask.astype(bool)
             table = table.loc[mask].copy()
-            
+        
+        # test to add function to a instance (does not require class)
+        table.graph = types.MethodType( plot_query_as_graph, table )
         
         return table
     
@@ -168,11 +172,13 @@ class Database():
     def clearLogTables(self):
         core.LOG['tableIDs'] = list()
 
-        
+    def isSource(self, sourceID):
+        return self.gitManager.isSource(sourceID)
 
     def commitTable(self, dataTable, message, sourceMetaDict=None):
-            
-        if dataTable.meta['source'] not in self.gitManager.sources.index:
+        
+        sourceID = dataTable.meta['source']
+        if not self.isSource(sourceID):
             if sourceMetaDict is None:
                 raise(BaseException('Source does not extist and now sourceMeta provided'))
             else:
@@ -194,7 +200,7 @@ class Database():
                      overwrite=False , 
                      cleanTables=True):
         # create a new source if not extisting
-        if not(sourceMetaDict['SOURCE_ID'] in self.gitManager.sources.index):
+        if not self.isSource(sourceMetaDict['SOURCE_ID']):
             self._addNewSource(sourceMetaDict)
 
         # only test if an table is update if the source did exist
@@ -233,7 +239,11 @@ class Database():
         
 
     def updateTable(self, oldTableID, newDataTable, message):
-
+        
+        sourceID = self._getSourceFromID(newDataTable.ID)
+        if not self.isSource(sourceID):
+            raise(BaseException('source  does not exist'))
+            
         self._updateTable(oldTableID, newDataTable)
         self._gitCommit(message)
     
@@ -242,6 +252,18 @@ class Database():
         """
         same as updateTable, but for list of tables
         """
+        sourcesToUpdate = list()
+        for tableID in oldTableIDs:
+            sourceID = self._getSourceFromID(tableID)
+            if sourceID not in sourcesToUpdate:
+                sourcesToUpdate.append(sourceID)
+        
+        # check that all sources do exist
+        for source in sourcesToUpdate:
+            if not self.isSource(sourceID):
+                raise(BaseException('source  does not exist'))
+        
+        
         for oldTableID, newDataTable in tqdm.tqdm(zip(oldTableIDs, newDataTables)):
             
             if oldTableID in self.inventory.index:
@@ -261,7 +283,7 @@ class Database():
         #print(oldDataTable.meta)
         newID = newDataTable.generateTableID()
         
-        if oldID == newID:
+        if oldID == newID and (oldDataTable.meta['unit'] == newDataTable.meta['unit']):
             #only change data
             self._addTable( newDataTable)
         else:
@@ -282,7 +304,7 @@ class Database():
         GREEN = '\033[32m'
         BLACK = '\033[30m'
         source = ID.split(config.ID_SEPARATOR)[-1]
-
+        print('TableID: {}'.format(ID))
         valid = list()
         if self.sourceExists(source):
             if print_statement:
@@ -319,23 +341,34 @@ class Database():
         return all(valid)
 
     def removeTables(self, IDList):
+        
+        sourcesToUpdate = list()
+        for tableID in IDList:
+            sourceID = self._getSourceFromID(tableID)
+            if sourceID not in sourcesToUpdate:
+                sourcesToUpdate.append(sourceID)
+        
+        # check that all sources do exist
+        for source in sourcesToUpdate:
+            if not self.isSource(sourceID):
+                raise(BaseException('source  does not exist'))
+        
         for ID in IDList:
             source = self.inventory.loc[ID, 'source']
             tablePath = self._getTableFilePath(ID)
-#            try:
+
             self.remove_from_inventory(ID)
-#            except:
-#                print('ID:' + ID +' not in inventory')
-#            try:
-#            self.gitManager[source].execute(["git", "rm", tablePath])
             self.gitManager.gitRemoveFile(source, tablePath)
-#            except:
-#                print('could not delete file:' + str(tablePath))
+
         self._gitCommit('Tables removed')
         
-    def removeTable(self, ID):
+    def removeTable(self, tableID):
 
-        self._removeTable( ID)
+        sourceID = self._getSourceFromID(tableID)
+        if not self.isSource(sourceID):
+            raise(BaseException('source  does not exist'))
+        
+        self._removeTable( tableID)
         self._gitCommit('Table removed')
         self._reloadInventory()
 
@@ -362,21 +395,23 @@ class Database():
     def isConsistentTable(self, datatable):
         
         if not pd.np.issubdtype(datatable.values.dtype, pd.np.number):
-            raise(BaseException('Sorry, data is needed to be numeric'))            
+            
+            raise(BaseException('Sorry, data of table {} is needed to be numeric'.format(datatable)))            
             
         # check that spatial index is consistend with defined countries or regions
         for regionIdx in datatable.index:
             #print(regionIdx + '-')
             if not mapp.regions.exists(regionIdx) and not mapp.countries.exists(regionIdx):
-                raise(BaseException('Sorry, region: ' + regionIdx + ' does not exist'))
+                raise(BaseException('Sorry, region in table {}: ' + regionIdx + ' does not exist'.format(datatable)))
         
         # check that the time colmns are years
         from pandas.api.types import is_integer_dtype
         if not is_integer_dtype(datatable.columns):
-            raise(BaseException('Sorry, year index is not integer'))
+            raise(BaseException('Sorry, year index in table {} is not integer'.format(datatable)))
 
         if sum(datatable.index.duplicated()) > 0:
-            raise(BaseException('Sorry, region index is unique'))
+            print(datatable.meta)
+            raise(BaseException('Sorry, region index in table {} is not  unique'.format(datatable)))
         return True
     
 
@@ -407,14 +442,15 @@ class Database():
         #self.add_to_inventory(datatable)
         
     def _gitAddTable(self, datatable, source, filePath):
+        
         datatable.to_csv(os.path.join(config.PATH_TO_DATASHELF, filePath))
         
         self.gitManager.gitAddFile(source, os.path.join('tables', self._getTableFileName(datatable.ID)))
         
     def _gitCommit(self, message):
-        self.inventory.to_csv(self.INTVENTORY_PATH)
-#        self['main'].execute(["git", "add", self.INTVENTORY_PATH])
-        self.gitManager.gitAddFile('main',self.INTVENTORY_PATH)
+        self.inventory.to_csv(self.INVENTORY_PATH)
+#        self['main'].execute(["git", "add", self.INVENTORY_PATH])
+        self.gitManager.gitAddFile('main',self.INVENTORY_PATH)
 
         for sourceID in self.gitManager.updatedRepos:
             repoPath = os.path.join(config.PATH_TO_DATASHELF, 'database', sourceID)
@@ -436,6 +472,10 @@ class Database():
         else:
             print('source already exists')
 
+    def _getSourceFromID(self, tableID):
+        return tableID.split(config.ID_SEPARATOR)[-1]
+    
+    
     def removeSource(self, sourceID):
         import shutil
         if self.sourceExists(sourceID):
@@ -509,7 +549,7 @@ class Database():
         self.sources.loc[remoteName,'git_commit_hash'] = self.gitManager[remoteName].execute(['git', 'rev-parse', 'HEAD'])
         self.sources.to_csv(config.SOURCE_FILE)
         self.gitManager.gitAddFile('main', config.SOURCE_FILE) 
-        sourceInventory = pd.read_csv(os.path.join(repoPath, 'source_inventory.csv'), index_col=0)
+        sourceInventory = pd.read_csv(os.path.join(repoPath, 'source_inventory.csv'), index_col=0, dtype={'source_year': str})
         for idx in sourceInventory.index:
             self.inventory.loc[idx,:] = sourceInventory.loc[idx,:]
         self._gitCommit('imported ' + remoteName)
@@ -678,4 +718,15 @@ class GitRepository_Manager(dict):
         
     def updateGitHash(self, repoName):
         self.sources.loc[repoName,'git_commit_hash'] = self[repoName].execute(['git', 'rev-parse', 'HEAD'])
-        self.commit()
\ No newline at end of file
+        self.commit()
+        
+    def setActive(self, repoName):
+        self[repoName].refresh()
+        
+    def isSource(self, sourceID):
+        if sourceID in self.sources.index:
+            self[sourceID].refresh()
+            return True
+        else:
+            return False
+        
\ No newline at end of file
