@@ -95,9 +95,61 @@ def read_MAGICC6_BINOUT(filePath):
 
     return Datatable(data, meta=metaDict)
 
+def read_PRIMAP_csv(fileName):
+    
+    metaMapping = {
+            'entity': 'SHEET_ENTITY',
+            'unit':'SHEET_UNIT',
+            'category':'SHEET_NAME_CATEGORY',
+            'scenario': 'SHEET_SCENARIO',
+            'model' : 'SHEET_SOURCE'
+            }
+    
+    allDf = pd.read_csv(fileName, usecols=[0,1], index_col=0)
+    #print(os.path.basename(fileName))
 
-def read_PRIMAP_Excel(fileName, sheet_name= 0):
-    allDf = pd.read_excel(fileName, sheet_name = sheet_name, usecols=[0,1], index_col=0)
+    firstDataRow = allDf.loc['SHEET_FIRSTDATAROW','Unnamed: 1']
+    
+    #bugfix of old wrong formated PRIMAP files
+    try:
+        int(firstDataRow)
+    except:
+        firstDataRow = pd.np.where(allDf.index == "Countries\Years")[0][0]+3
+   
+    firstMetaRow = pd.np.where(allDf.index == "&SHEET_SPECIFICATIONS")[0][0]+1
+    
+    metaPrimap = dict()
+    for row in range(firstMetaRow,firstDataRow):
+        key = allDf.index[row]
+        value = allDf.iloc[row,0]
+        if key =='/':
+            break
+        metaPrimap[key] = value
+    
+    data = pd.read_csv(fileName, header=firstDataRow-2, index_col=0)
+    
+    meta = dict()
+    for metaKey in metaMapping:
+        
+        if isinstance(metaMapping[metaKey], list):
+            value = '_'.join(metaPrimap[x] for x in metaMapping[metaKey])
+        else:
+            value = metaPrimap[metaMapping[metaKey]]
+        meta[metaKey] = value
+    
+    
+    
+    table = Datatable(data, meta=meta)
+    table = table.loc[:, util.yearsColumnsOnly(table)]
+    table.columns = table.columns.astype(int)
+    return table#, data
+
+def read_PRIMAP_Excel(fileName, sheet_name= 0, xlsFile=None):
+    if xlsFile is not None:
+        xlsFile = xlsFile
+    else:   
+        xlsFile = pd.ExcelFile(fileName)
+    allDf = pd.read_excel(xlsFile, sheet_name = sheet_name, usecols=[0,1], index_col=0)
     #print(os.path.basename(fileName))
 
     firstDataRow = allDf.loc['SHEET_FIRSTDATAROW','Unnamed: 1']
@@ -116,7 +168,7 @@ def read_PRIMAP_Excel(fileName, sheet_name= 0):
     setup['timeIdxList']  = ('B' + str(firstDataRow-1), 'XX'+ str(firstDataRow-1))
     setup['spaceIdxList'] = ('A'+ str(firstDataRow), 'A1000')
     #print(setup)
-    ex = ExcelReader(setup)
+    ex = ExcelReader(setup, xlsFile=xlsFile)
     data = ex.gatherData().astype(float)
     #return data
     meta = {'source': '',
@@ -132,7 +184,9 @@ def read_PRIMAP_Excel(fileName, sheet_name= 0):
     
     table = Datatable(data, meta=meta)
     try:
+        table = table.loc[:,util.yearsColumnsOnly(table)]
         table.columns = table.columns.astype(int)
+#        table = table.loc[:,util.yearsColumnsOnly(table)]
     except:
         print('warning: Columns could not be converted to int')
     return table
@@ -192,8 +246,11 @@ def read_MAGICC6_ScenFile(fileName, **kwargs):
     # TODO: CHange to a results list of datatables
     return entityFrame
 
-def insertDataIntoExcelFile(fileName, overwrite = False, setupSheet = 'INPUT'):
-    ins = ExcelWriter(fileName=fileName, overwrite = overwrite, setupSheet = setupSheet)
+def insertDataIntoExcelFile(fileName, 
+                            overwrite = False, 
+                            setupSheet = 'INPUT',
+                            interpolate = False):
+    ins = ExcelWriter(fileName=fileName, overwrite = overwrite, setupSheet = setupSheet, interpolate=interpolate)
     ins.insert_data()
     ins.close()
     return ins
@@ -224,7 +281,7 @@ def getAllSheetNames(filePath):
 
 class ExcelReader():
     
-    def __init__(self, setupDict):
+    def __init__(self, setupDict, xlsFile=None):
         """
         Required setup parameters:
             filePath    = './data/'
@@ -235,6 +292,8 @@ class ExcelReader():
         
         """
         self.setup(**setupDict)
+        if xlsFile is not None:
+            self.xlsFile = xlsFile
         
     
 #    def setSetup(self, **kwargs):
@@ -268,7 +327,10 @@ class ExcelReader():
 
     def gatherValue(self, excelIdx):
         if not hasattr(self, 'df'):
-            self.df = pd.read_excel(os.path.join(self.filePath, self.fileName), sheet_name=self.sheetName, header=None)
+            if hasattr(self, 'xlsFile'):
+                self.df = pd.read_excel(self.xlsFile, sheet_name=self.sheetName, header=None)
+            else:
+                self.df = pd.read_excel(os.path.join(self.filePath, self.fileName), sheet_name=self.sheetName, header=None)
         elif core.config.DEBUG:
             print('use loaded df')
         return self.df.iloc[excelIdx2PandasIdx(excelIdx)]             
@@ -294,7 +356,7 @@ class ExcelReader():
         if len(self.timeIdxList) >1:
             timeIdx = self.df.iloc[self.timeIdxList[0][0],self.timeIdxList[0][1]:self.timeIdxList[1][1]+1]
             columns = self.df.columns[self.timeIdxList[0][1]:self.timeIdxList[1][1]+1]
-            print(timeIdx)
+#            print(timeIdx)
         else:
             timeIdx = self.df.iloc[self.timeIdxList[0][0],[self.timeIdxList[0][1]]]
             columns = self.df.columns[[self.timeIdxList[0][1]]]
@@ -586,7 +648,11 @@ class ExcelWriter():
     More complex excel interface
     Author: Andreas Geiges
     """
-    def __init__(self, setup=None, fileName=None, overwrite=False, setupSheet = 'INPUT'):
+    def __init__(self, setup=None, 
+                 fileName=None, 
+                 overwrite=False, 
+                 setupSheet = 'INPUT',
+                 interpolate=False):
         self.overwrite = overwrite # overwriting values
         self.setupSheet= setupSheet
         if setup:
@@ -597,7 +663,7 @@ class ExcelWriter():
             self.setup['fileName'] = fileName
             fileSetup = False
     
-
+        self.interpolate = interpolate
     
     def close(self):
         self.wb.close()
@@ -706,7 +772,14 @@ class ExcelWriter():
                  for tableKey in tables.keys():
                      table = tables[tableKey]
                      tables[tableKey]= table.convert(setup['unit'])
-            
+                     
+            if self.interpolate:
+                 for tableKey in tables.keys():
+                     for col in list(range(tables[tableKey].columns.min(),tables[tableKey].columns.max()+1)):
+                         if col not in tables[tableKey].columns:
+                             tables[tableKey].loc[:,col] = np.nan
+                     tables[tableKey] = tables[tableKey].interpolate()    
+                     
 #            wksSheet = load_workbook(setup['fileName'], data_only=True)[setup['sheetName']]
             
             args =  [None, None, None, None, None]
